/*
 * Copyright (c) 1998,1999,2000,2003 Damien Miller.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/* 
 * Routines and data structures used to store and manage packet 
 * summary report
 */

#include <stdlib.h>
#include <stdio.h>
#include <netdb.h>
#include <time.h>
#include <netinet/in.h>
#include <sys/types.h>
#include <sys/socket.h>

#include "report.h"
#include "util.h"

static char rcsid[] = "$Id: report.c,v 1.5 2003/06/20 07:07:17 djm Exp $";

/* Prototypes */
char *resolve_hostname(u_int32_t ip_addr);

void free_report(report_t *r)
{
	free_hosts(r->hosts);
	free(r);
}

void free_host(host_t *h)
{
	if (h->hostname != NULL)
		free(h->hostname);
	free(h);
}

void free_hosts(host_t *h)
{
	host_t *last_h;

	while (h != NULL)
	{
		last_h = h;
		h = h->next;

		free_peers(last_h->peers);
		free_host(last_h);
	}
}

void free_peer(peer_t *p)
{
	free(p);
}

void free_peers(peer_t *p)
{
	peer_t *last_p;

	while (p != NULL)
	{
		last_p = p;
		p = p->next;

		free_peer(last_p);
	}
}

/* Hashes a host's ip address */
unsigned int host_hash(u_int32_t *ip_addr)
{
	return((unsigned int)(*ip_addr));
}

/* Compares two hosts */
unsigned int host_compare(u_int32_t *ip_addr1, u_int32_t *ip_addr2)
{
	return((*ip_addr1) == (*ip_addr2));
}

/* Resolves all DNS names in host list 'h' */
void resolve_hosts(host_t *h)
{
	while(h != NULL)
	{
		if (h->hostname != NULL)
			free(h->hostname);
			
		h->hostname = resolve_hostname(h->ip_addr);
		
		h = h->next;
	}
}

/* attempts to look up DNS name for ip_addr */
/* returns NULL if no name found */
/* returns name in malloc'd buffer if found */
char *resolve_hostname(u_int32_t ip_addr)
{
	struct hostent *h;
	
	if (ip_addr == INADDR_ANY)
		return(util_strdup("ALL-ZEROS"));

	if (ip_addr == INADDR_BROADCAST)
		return(util_strdup("ALL-ONES"));

	h = gethostbyaddr((char *)&ip_addr, sizeof(ip_addr), AF_INET);
	
	if ((h == NULL) || (h->h_name == NULL))
		return(NULL);
		
	return(util_strdup(h->h_name));
}
