/*
 * Copyright (c) 1998,1999,2000,2003 Damien Miller.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/* Miscellaneous utility routines */

#include <stdlib.h>
#include <unistd.h>
#include <stdio.h>
#include <string.h>
#include <signal.h>
#include <time.h>
#include <netdb.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <sys/types.h>
#include <math.h>

#include <glib.h>

#include "util.h"

static char rcsid[] = "$Id: util.c,v 1.6 2003/06/20 07:07:17 djm Exp $";

/* Allocates s bytes, checking for allocation errors */
void *util_malloc(size_t s)
{
	void *m;
	
	m = g_malloc(s);

	if (m == NULL)
	{
		fprintf(stderr, "\nOut of memory.\n");
		raise(SIGSEGV);
	}
	
	return(m);
}

/* Allocates s zero-filled bytes, checking for allocation errors */
void *util_zmalloc(size_t s)
{
	void *m;
	
	m = g_malloc(s);

	if (m == NULL)
	{
		fprintf(stderr, "\nOut of memory.\n");
		raise(SIGSEGV);
	}
	
	memset(m, 0, s);
	
	return(m);
}

/* Duplicates string s, checking for allocation errors */
void *util_strdup(const char *s)
{
	char *m;
	
	m = g_strdup(s);

	if (m == NULL)
	{
		fprintf(stderr, "\nOut of memory.\n");
		raise(SIGSEGV);
	}
	
	return(m);
}

/* Resizes allocation m to s bytes, checking for allocation errors */
void *util_realloc(void *m, size_t s)
{
	m = g_realloc(m, s);

	if (m == NULL)
	{
		fprintf(stderr, "\nOut of memory.\n");
		raise(SIGSEGV);
	}
	
	return(m);
}

/* Seeds RNG  */
void util_seed_rng(void)
{
	srandom(time(NULL) ^ (getpid() + (getppid() << 15)));
}

/* Parses an address and mask */
int parse_address_and_mask(const char *addrspec, u_int32_t *address, u_int32_t *mask)
{
	char address_str[20];
	char mask_str[20];
	struct in_addr i_addr;
	struct hostent *h;
	int cidr_bits;
	int c;
	
	/* Check and split address specification */
	c = sscanf(addrspec, "%18[0-9.]/%18[0-9.]", address_str, mask_str);
	
	/* If no values were converted, try to look up name in DNS */
	if (c == 0)
	{
		h = gethostbyname(addrspec);
		
		/* If no host of suitable type found, return an error */
		if ((h == NULL) || (h->h_addrtype != AF_INET))
			return(0);
		
		memcpy(address, h->h_addr_list[0], sizeof(*address));
		*mask = 0xFFFFFFFF;
		
		return(1);
	}

	/* Convert address part, returning an error if conversion failed */
	if (inet_aton(address_str, &i_addr) == 0)
		return(0);
	*address = i_addr.s_addr;

	/* If only address was specified, return it with all-ones mask */
	if (c == 1)
	{
		*mask = 0xFFFFFFFF;
		return(1);
	}

	/* Check whether a CIDR network-length was specified */
	if (strchr(mask_str, '.') == NULL)
	{
		cidr_bits = atoi(mask_str);

		/* Check for out of bounds CIDR bits */
		if ((cidr_bits < 0) || (cidr_bits > 32))
			return(0);

		/* Check for special case of num. bits == width of address */
		/* As the nice mask calculation formula doesn't work in that case */
		if (cidr_bits == 32)
			*mask = 0xFFFFFFFF;
		else 
			*mask = htonl(0xFFFFFFFF ^ (0xFFFFFFFF >> cidr_bits));
			
		return(1);
	}

	/* Otherwise, assume a netmask was specified */
	/* Attempt to convert it, returning an error if conversion failed */
	if (inet_aton(mask_str, &i_addr) == 0)
		return(0);
	*mask = i_addr.s_addr;
	
	return(1);
}
